Package: torchdatasets
Title: Ready to Use Extra Datasets for Torch
Version: 0.3.2
Authors@R: 
    c(
    person("Tomasz", "Kalinowski", role = c("ctb", "cre"), email = "tomasz@posit.co"),
    person("Daniel", "Falbel", role = c("aut", "cph"), email = "dfalbel@gmail.com"),
    person(family = "RStudio", role = c("cph"))
    )
Description: Provides datasets in a format that can be easily consumed by torch 'dataloaders'.
  Handles data downloading from multiple sources, caching and pre-processing so
  users can focus only on their model implementations.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: torch (>= 0.5.0), fs, zip, pins, torchvision, stringr, withr,
        utils
Suggests: testthat, readr, coro, tokenizers, R.matlab
URL: https://mlverse.github.io/torchdatasets/,
        https://github.com/mlverse/torchdatasets
BugReports: https://github.com/mlverse/torchdatasets/issues
NeedsCompilation: no
Packaged: 2026-04-28 16:21:17 UTC; dfalbel
Author: Tomasz Kalinowski [ctb, cre],
  Daniel Falbel [aut, cph],
  RStudio [cph]
Maintainer: Tomasz Kalinowski <tomasz@posit.co>
Repository: CRAN
Date/Publication: 2026-04-29 06:40:02 UTC
Built: R 4.4.3; ; 2026-05-01 14:02:45 UTC; unix

# Generated by roxygen2: do not edit by hand

export(bank_marketing_dataset)
export(bird_species_dataset)
export(cityscapes_pix2pix_dataset)
export(dogs_vs_cats_dataset)
export(guess_the_correlation_dataset)
export(imdb_dataset)
export(oxford_flowers102_dataset)
export(oxford_pet_dataset)
export(spam_dataset)
importFrom(stringr,str_extract)
importFrom(torch,dataset)
importFrom(torchvision,base_loader)

# torchdatasets

<!-- badges: start -->

[![R-CMD-check](https://github.com/mlverse/torchdatasets/workflows/R-CMD-check/badge.svg)](https://github.com/mlverse/torchdatasets/actions) [![CRAN status](https://www.r-pkg.org/badges/version/torchdatasets)](https://CRAN.R-project.org/package=torchdatasets) [![](https://cranlogs.r-pkg.org/badges/torchdatasets)](https://cran.r-project.org/package=torchdatasets)

<!-- badges: end -->

torchdatasets provides ready-to-use datasets compatible with the [torch](https://github.com/mlverse/torch) package.

## Installation

The released version of torchdatasets can be installed with:

``` r
install.packages("torchdatasets")
```

You can also install the development version with:

``` r
remotes::install_github("mlverse/torchdatasets")
```

## Datasets

Currently, the following datasets are implemented:

| Dataset                         | Domain  | Type           | Authentication |
|---------------------------------|---------|----------------|----------------|
| bird_species_dataset()          | Images  | Classification | Not required   |
| dogs_vs_cats_dataset()          | Images  | Classification | Not required   |
| guess_the_correlation_dataset() | Images  | Regression     | Not required   |
| cityscapes_pix2pix_dataset()    | Images  | Segmentation   | Not required   |
| oxford_pet_dataset()            | Images  | Segmentation   | Not required   |
| bank_marketing_dataset()        | Tabular | Classification | Not required   |
| imdb_dataset()                  | Text    | Classification | Not required   |
| spam_dataset()                  | Tabular | Classification | Not required   |

#' Bird species dataset
#'
#'
#' Downloads and prepares the 450 bird species dataset found on Kaggle.
#'
#'
#' @param root path to the data location
#' @param split train, test or valid
#' @param download wether to download or not
#' @param ... other arguments passed to [torchvision::image_folder_dataset()].
#'
#' @return A [torch::dataset()] ready to be used with dataloaders.
#'
#' @examples
#' if (torch::torch_is_installed() && FALSE) {
#' birds <- bird_species_dataset("./data", token = "path/to/kaggle.json",
#'                               download = TRUE)
#' length(birds)
#' }
#' @export
bird_species_dataset <- torch::dataset(
  inherit = torchvision::image_folder_dataset,
  initialize = function(root, split = "train", download = FALSE, ...) {

    url <- "https://torch-cdn.mlverse.org/datasets/bird-species.zip"
    data_path <- maybe_download(
      root = root,
      name = "bird-species",
      url = url,
      download = download,
      extract_fun = function(temp, data_path) {
        unzip2(temp, exdir = data_path)
      }
    )

    if (!fs::dir_exists(data_path))
      cli::cli_abort("No data found. Please use `download = TRUE`.")

    possible_splits <- c("train", "valid", "test")
    if (!split %in% possible_splits) {
      cli::cli_abort(c(
        "Found split {.val {split}} but expected one of {.or {.val {possible_splits}}}."
      ))
    }

    p <- fs::path(data_path, split)
    super$initialize(root = p, ...)
  }
)

#' Oxford Pet Dataset
#'
#' The Oxford-IIIT Pet Dataset is a 37 category pet dataset with roughly
#' 200 images for each class. The images have a large variations in scale,
#' pose and lighting. All images have an associated ground truth annotation of
#' species (cat or dog), breed, and pixel-level trimap segmentation.
#'
#' @inheritParams cityscapes_pix2pix_dataset
#' @param target_type The type of the target:
#'   - 'trimap': returns a mask array with one class per pixel.
#'   - 'species': returns the species id. 1 for cat and 2 for dog.
#'   - 'breed': returns the breed id. see `dataset$breed_classes`.
#'
#' @export
oxford_pet_dataset <- torch::dataset(
  "OxfordPet",

  trimap_classes = c("Pixel belonging to the pet", "Pixel bordering the pet", "Surrounding pixel"),
  species_classes = c("cat", "dog"),
  breed_classes = c("Abyssinian", "american_bulldog", "american_pit_bull_terrier",
                    "basset_hound", "beagle", "Bengal", "Birman", "Bombay", "boxer",
                    "British_Shorthair", "chihuahua", "Egyptian_Mau", "english_cocker_spaniel",
                    "english_setter", "german_shorthaired", "great_pyrenees", "havanese",
                    "japanese_chin", "keeshond", "leonberger", "Maine_Coon", "miniature_pinscher",
                    "newfoundland", "Persian", "pomeranian", "pug", "Ragdoll", "Russian_Blue",
                    "saint_bernard", "samoyed", "scottish_terrier", "shiba_inu",
                    "Siamese", "Sphynx", "staffordshire_bull_terrier", "wheaten_terrier",
                    "yorkshire_terrier"),

  initialize = function(root, split = "train", target_type = c("trimap", "species", "breed"),
                        download = FALSE, ..., transform = NULL, target_transform = NULL) {

    rlang::check_installed("readr")

    data_path <- fs::path_expand(fs::path(root, "oxford-pet"))
    self$data_path <- data_path

    if (!fs::dir_exists(data_path) && download) {

      images <- download_file(
        "https://www.robots.ox.ac.uk/~vgg/data/pets/data/images.tar.gz",
        tempfile(fileext = ".tar.gz")
      )

      targets <- download_file(
        "https://www.robots.ox.ac.uk/~vgg/data/pets/data/annotations.tar.gz",
        tempfile(fileext = ".tar.gz")
      )

      fs::dir_create(data_path)
      untar(images, exdir = data_path)
      untar(targets, exdir = data_path)
    }

    if (!fs::dir_exists(data_path))
      stop("No data found. Please use `download = TRUE`.")

    self$split <- split
    self$target_type <- rlang::arg_match(target_type)
    self$target_reader <- get(
      paste0("read_", self$target_type),
      envir = self
    )
    self$classes <- get(
      paste0(self$target_type, "_classes"),
      envir = self
    )

    if (self$split == "train") {
      img_list <- fs::path(data_path, "annotations", "trainval.txt")
    } else {
      img_list <- fs::path(data_path, "annotations", "test.txt")
    }

    self$imgs <- readr::read_delim(
      img_list,
      delim = " ",
      col_names = c("image", "class_id", "species_id", "breed_id"),
      col_types = readr::cols()
    )

    self$transform <- if (is.null(transform)) identity else transform
    self$target_transform <- if (is.null(target_transform)) identity else target_transform

    # rename files known to be PNG's
    self$imgs$ext <- "jpg"
    pngs <- c("Egyptian_Mau_14", "Egyptian_Mau_156", "Egyptian_Mau_186", "Abyssinian_5")
    self$imgs$ext[self$imgs$image %in% pngs] <- "png"

    # remove corrupt file
    self$imgs <- self$imgs[self$imgs$image != "beagle_116",]

  },

  .getitem = function(i) {
    img <- self$imgs[i,]
    list(
      x = self$transform(self$read_img(img)),
      y = self$target_transform(self$target_reader(img))
    )
  },

  .length = function() {
    nrow(self$imgs)
  },

  read_img = function(img) {
    path <- fs::path(self$data_path, "images", paste0(img$image, ".jpg"))
    if (img$ext == "jpg")
      jpeg::readJPEG(path)
    else
      png::readPNG(path)[,,1:3] # we remove the alpha channel
  },

  read_trimap = function(img) {
    mask <- png::readPNG(fs::path(self$data_path, "annotations", "trimaps", paste0(img$image, ".png")))
    dimensions <- dim(mask)
    mask <- as.integer(mask*255)
    dim(mask) <- dimensions
    mask
  },

  read_species = function(img) {
    as.integer(img$species_id)
  },

  read_breed = function(img) {
    as.integer(img$breed_id)
  }

)

test_that("bank marketting works", {

  data <- bank_marketing_dataset(
    root = tempfile(),
    download = TRUE
  )

  expect_length(data$.getitem(1), 2)

  dl <- torch::dataloader(data, batch_size = 32)
  x <- coro::collect(dl, n = 1)

  expect_equal(x[[1]]$x$shape, c(32, 55))
  expect_equal(x[[1]]$y$shape, c(32))

})

test_that("guess_the_correlation_dataset works", {

  tmp <- tempfile()
  dataset <- guess_the_correlation_dataset(
    root = tmp,
    download = TRUE,
    transform = function(x) torch::torch_zeros(3,3)
  )

  expect_length(dataset$.getitem(1), 3)
  expect_equal(dim(dataset$.getitem(1)$x), c(3,3))
  expect_true(dataset$.getitem(1)$y$dtype == torch::torch_float())

})

flower_categories <- c(
  "pink primrose",
  "hard-leaved pocket orchid",
  "canterbury bells",
  "sweet pea",
  "english marigold",
  "tiger lily",
  "moon orchid",
  "bird of paradise",
  "monkshood",
  "globe thistle",
  "snapdragon",
  "colt's foot",
  "king protea",
  "spear thistle",
  "yellow iris",
  "globe-flower",
  "purple coneflower",
  "peruvian lily",
  "balloon flower",
  "giant white arum lily",
  "fire lily",
  "pincushion flower",
  "fritillary",
  "red ginger",
  "grape hyacinth",
  "corn poppy",
  "prince of wales feathers",
  "stemless gentian",
  "artichoke",
  "sweet william",
  "carnation",
  "garden phlox",
  "love in the mist",
  "mexican aster",
  "alpine sea holly",
  "ruby-lipped cattleya",
  "cape flower",
  "great masterwort",
  "siam tulip",
  "lenten rose",
  "barbeton daisy",
  "daffodil",
  "sword lily",
  "poinsettia",
  "bolero deep blue",
  "wallflower",
  "marigold",
  "buttercup",
  "oxeye daisy",
  "common dandelion",
  "petunia",
  "wild pansy",
  "primula",
  "sunflower",
  "pelargonium",
  "bishop of llandaff",
  "gaura",
  "geranium",
  "orange dahlia",
  "pink-yellow dahlia?",
  "cautleya spicata",
  "japanese anemone",
  "black-eyed susan",
  "silverbush",
  "californian poppy",
  "osteospermum",
  "spring crocus",
  "bearded iris",
  "windflower",
  "tree poppy",
  "gazania",
  "azalea",
  "water lily",
  "rose",
  "thorn apple",
  "morning glory",
  "passion flower",
  "lotus",
  "toad lily",
  "anthurium",
  "frangipani",
  "clematis",
  "hibiscus",
  "columbine",
  "desert-rose",
  "tree mallow",
  "magnolia",
  "cyclamen",
  "watercress",
  "canna lily",
  "hippeastrum",
  "bee balm",
  "ball moss",
  "foxglove",
  "bougainvillea",
  "camellia",
  "mallow",
  "mexican petunia",
  "bromelia",
  "blanket flower",
  "trumpet creeper",
  "blackberry lily"
)

#' 102 Category Flower Dataset
#'
#' The Oxford Flower Dataset is a 102 category dataset, consisting of 102 flower
#' categories. The flowers chosen to be flower commonly occuring in the United
#' Kingdom. Each class consists of between 40 and 258 images. The details of the
#' categories and the number of images for each class can be found on
#' [this category statistics page](https://www.robots.ox.ac.uk/%7Evgg/data/flowers/102/categories.html).
#'
#' The images have large scale, pose and light variations. In addition, there are
#' categories that have large variations within the category and several very
#' similar categories. The dataset is visualized using isomap with shape and colour
#' features.
#'
#' You can find more info in the dataset [webpage](https://www.robots.ox.ac.uk/%7Evgg/data/flowers/102/).
#'
#' @note The official splits leaves far too many images in the test set. Depending
#'   on your work you might want to create different train/valid/test splits.
#'
#' @inheritParams oxford_pet_dataset
#' @param target_type Currently only 'categories' is supported.
#' @importFrom torch dataset
#' @export
oxford_flowers102_dataset <- torch::dataset(
  "OxfordFlowers102",
  classes = flower_categories,
  initialize = function(root, split = "train", target_type = c("categories"),
                        download = FALSE, ..., transform = NULL, target_transform = NULL) {
    rlang::check_installed(c("R.matlab"))

    data_path <- fs::path_expand(fs::path(root, "oxford-flowers102"))
    self$data_path <- data_path

    if (!fs::dir_exists(data_path) && download) {

      images <- download_file(
        "https://torch-cdn.mlverse.org/datasets/oxford_flowers102/102flowers.tgz",
        tempfile(fileext = ".tgz")
      )

      targets <- download_file(
        "https://torch-cdn.mlverse.org/datasets/oxford_flowers102/imagelabels.mat",
        tempfile(fileext = ".mat"),
        mode = "wb"
      )

      splits <- download_file(
        "https://torch-cdn.mlverse.org/datasets/oxford_flowers102/setid.mat",
        tempfile(fileext = ".mat"),
        mode = "wb"
      )

      fs::dir_create(data_path)
      untar(images, exdir = data_path)
      fs::file_move(targets, fs::path(data_path, "imagelabels.mat"))
      fs::file_move(splits, fs::path(data_path, "setid.mat"))
    }

    if (!fs::dir_exists(data_path))
      cli::cli_abort("No data found. Please use {.var download = TRUE}.")

    self$split <- split
    splits <- R.matlab::readMat(fs::path(self$data_path, "setid.mat"))
    splits <- lapply(splits, as.integer)
    names(splits) <- c("train", "valid", "test")

    self$target_type <- target_type
    targets <- R.matlab::readMat(fs::path(self$data_path, "imagelabels.mat"))
    targets <- as.integer(targets$labels)

    ids <- unlist(splits[names(splits) %in% self$split])
    self$targets <- targets[ids]

    self$imgs <- fs::path(
      self$data_path,
      "jpg",
      sprintf("image_%05d.jpg", ids)
    )

    self$transform <- if (is.null(transform)) identity else transform
    self$target_transform <- if (is.null(target_transform)) identity else target_transform
  },
  .getitem = function(i) {
    list(
      x = self$transform(jpeg::readJPEG(self$imgs[i])),
      y = self$target_transform(self$targets[i])
    )
  },
  .length = function() {
    length(self$imgs)
  }
)

kaggle_download <- function(name, token = NULL) {

  if ("kaggle" %in% pins::board_list()) {
    file <- pins::pin_get(board = "kaggle", name,
                          extract = FALSE)
  } else if (!is.null(token)) {
    pins::board_register_kaggle(name="torchdatasets-kaggle", token = token,
                                cache = tempfile(pattern = "dir"))
    on.exit({pins::board_deregister("torchdatasets-kaggle")}, add = TRUE)
    file <- pins::pin_get(name,
                          board = "torchdatasets-kaggle",
                          extract = FALSE)
  } else {
    stop("Please register the Kaggle board or pass the `token` parameter.")
  }

  file
}

download_file <- function(url, destfile, ...) {
  withr::with_options(new = list(timeout = max(600, getOption("timeout"))), {
    utils::download.file(url, destfile, ...)
  })
  destfile
}

maybe_download <- function(url, root, name, extract_fun, download) {
  data_path <- fs::path_expand(fs::path(root, name))

  if (!fs::dir_exists(data_path) && download) {
    tmp <- tempfile()
    download_file(url, tmp)
    fs::dir_create(fs::path_dir(data_path), recurse = TRUE)
    extract_fun(tmp, data_path)
  }

  if (!fs::dir_exists(data_path))
    stop("No data found. Please use `download = TRUE`.")

  data_path
}

unzip2 <- function(path, exdir) {
  if (grepl("linux", R.version$os)) {
    utils::unzip(path, exdir = exdir)
  } else {
    zip::unzip(path, exdir = exdir)
  }
}

#' IMDB movie review sentiment classification dataset
#'
#' The format of this dataset is meant to replicate that provided by
#' [Keras](https://keras.io/api/datasets/imdb/).
#'
#' @inheritParams bird_species_dataset
#' @param shuffle whether to shuffle or not the dataset. `TRUE` if `split=="train"`
#' @param num_words Words are ranked by how often they occur (in the training set),
#'   and only the num_words most frequent words are kept. Any less frequent word
#'   will appear as oov_char value in the sequence data. If `Inf`, all words are
#'   kept. Defaults to None, so all words are kept.
#' @param skip_top skip the top N most frequently occurring words (which may not be informative).
#'   These words will appear as oov_char value in the dataset. Defaults to 0, so
#'   no words are skipped.
#' @param maxlen int or `Inf`. Maximum sequence length. Any longer sequence will
#'   be truncated. Defaults to Inf, which means no truncation.
#' @param start_char The start of a sequence will be marked with this character.
#'   Defaults to 2, because 1 is usually the padding character.
#' @param oov_char int. The out-of-vocabulary character. Words that were cut out
#'   because of the num_words or skip_top limits will be replaced with this character.
#' @param index_from int. Index actual words with this index and higher.
#
#'
#' @export
imdb_dataset <- torch::dataset(
  initialize = function(root, download = FALSE, split = "train", shuffle = (split == "train"),
                        num_words = Inf, skip_top = 0, maxlen = Inf,
                        start_char = 2, oov_char = 3, index_from = 4) {

    rlang::check_installed("tokenizers")

    url = "https://ai.stanford.edu/~amaas/data/sentiment/aclImdb_v1.tar.gz"
    data_path <- maybe_download(
      url = url,
      root = root,
      download = download,
      name = "imdb",
      extract_fun = function(tmp, expath) {
        untar(tmp, exdir = expath)
      }
    )
    self$data_path <- data_path

    if (!split %in% c("train", "test"))
      rlang::abort(paste0("Unknown split `", split, "`"))

    texts <- self$read_and_tokenize(split)
    response <- texts$response
    texts <- texts$texts

    vocabulary <- self$get_vocabulary()

    if (skip_top > 0)
      vocabulary <- vocabulary[-seq_len(skip_top)]

    if (num_words < length(vocabulary))
      vocabulary <- vocabulary[seq_len(num_words)]

    if (shuffle) {
      new_order <- sample.int(length(texts))
      texts <- texts[new_order]
      response <- response[new_order]
    }

    self$texts <- texts
    self$response <- response
    self$vocabulary <- vocabulary
    self$start_char <- start_char
    self$oov_char <- oov_char
    self$maxlen <- maxlen
    self$index_from <- index_from
  },
  .getitem = function(i) {
    words <- self$texts[[i]]

    # word indexes start at 1, but we want them to start from `index_from`
    int <- match(words, names(self$vocabulary)) + as.integer(self$index_from - 1)
    int[is.na(int)] <- as.integer(self$oov_char)
    int <- c(as.integer(self$start_char), int)

    if (is.finite(self$maxlen)) {
      if (length(int) >= self$maxlen) {
        int <- int[seq_len(self$maxlen)]
      } else {
        int <- c(rep(1L, self$maxlen - length(int)), int)
      }
    }

    list(
      x = int,
      y = self$response[i]
    )
  },
  .length = function() {
    length(self$texts)
  },
  get_vocabulary = function() {

    data_path <- self$data_path
    cached <- fs::path(data_path, "aclImdb", "cached-vocab.rds")
    if (!fs::file_exists(cached)) {
      texts <- self$read_and_tokenize("train")$texts
      vocabulary <- texts %>%
        unlist() %>%
        table() %>%
        sort(decreasing = TRUE)
      saveRDS(vocabulary, file = cached)
    } else {
      vocabulary <- readRDS(cached)
    }

    vocabulary
  },
  read_and_tokenize = function(split) {

    data_path <- self$data_path
    cached <- fs::path(data_path, "aclImdb", split, "cached.rds")

    if (!fs::file_exists(cached)) {

      pos <- fs::dir_ls(fs::path(data_path, "aclImdb", split, "pos"))
      neg <- fs::dir_ls(fs::path(data_path, "aclImdb", split, "neg"))

      texts <- sapply(c(pos, neg), function(x) readr::read_file(x)) %>%
        tokenizers::tokenize_words()

      response <- c(
        rep(1, length.out = length(pos)),
        rep(0, length.out = length(neg))
      )

      rlang::inform(paste0("Caching tokenized texts for split: ", split))
      saveRDS(
        list(texts = texts, response = response),
        file = cached
      )
    } else {
      texts <- readRDS(cached)

      response <- texts$response
      texts <- texts$texts
    }

    list(
      texts = texts,
      response = response
    )
  }
)

#' Guess The Correlation dataset
#'
#' Prepares the Guess The Correlation dataset available on Kaggle [here](https://www.kaggle.com/c/guess-the-correlation)
#' A copy of this dataset is hosted in a public Google Cloud
#' bucket so you don't need to authenticate.
#'
#' @param root path to the data location
#' @param split string. 'train' or 'submission'
#' @param transform function that takes a torch tensor representing an image and return another tensor, transformed.
#' @param target_transform function that takes a scalar torch tensor and returns another tensor, transformed.
#' @param indexes set of integers for subsampling (e.g. 1:140000)
#' @param download whether to download or not
#'
#' @return A torch dataset that can be consumed with [torch::dataloader()].
#' @examples
#' if (torch::torch_is_installed() && FALSE) {
#' gtc <- guess_the_correlation_dataset("./data", download = TRUE)
#' length(gtc)
#' }
#' @export
guess_the_correlation_dataset <- torch::dataset(
  "GuessTheCorrelation",
  initialize = function(root, split = "train", transform = NULL, target_transform = NULL, indexes = NULL, download = FALSE) {

    self$transform <- transform
    self$target_transform <- target_transform

    # donwload ----------------------------------------------------------
    data_path <- maybe_download(
      root = root,
      name = "guess-the-correlation",
      url = "https://torch-cdn.mlverse.org/datasets/guess-the-correlation.zip",
      download = download,
      extract_fun = function(temp, data_path) {
        unzip2(temp, exdir = data_path)
        unzip2(fs::path(data_path, "train_imgs.zip"), exdir = data_path)
        unzip2(fs::path(data_path, "test_imgs.zip"), exdir = data_path)
      }
    )

    # variavel resposta -------------------------------------------------

    if(split == "train") {
      self$images <- readr::read_csv(fs::path(data_path, "train.csv"), col_types = c("cn"))
      if(!is.null(indexes)) self$images <- self$images[indexes, ]
      self$.path <- file.path(data_path, "train_imgs")
    } else if(split == "submission") {
      self$images <- readr::read_csv(fs::path(data_path, "example_submition.csv"), col_types = c("cn"))
      self$images$corr <- NA_real_
      self$.path <- file.path(data_path, "test_imgs")
    }
  },

  .getitem = function(index) {

    force(index)

    sample <- self$images[index, ]
    id <- sample$id
    x <- torchvision::base_loader(file.path(self$.path, paste0(sample$id, ".png")))
    x <- torchvision::transform_to_tensor(x) %>% torchvision::transform_rgb_to_grayscale()

    if (!is.null(self$transform))
      x <- self$transform(x)

    y <- torch::torch_scalar_tensor(sample$corr)
    if (!is.null(self$target_transform))
      y <- self$target_transform(y)

    return(list(x = x, y = y, id = id))
  },

  .length = function() {
    nrow(self$images)
  }
)

#' Bank marketing dataset
#'
#' Prepares the Bank marketing dataset available on UCI Machine Learning repository [here](https://archive.ics.uci.edu/ml/datasets/Bank+Marketing)
#' The data is available publicly for download, there is no need to authenticate.
#' Please cite the data as Moro et al., 2014
#' S. Moro, P. Cortez and P. Rita. A Data-Driven Approach to Predict the Success of Bank Telemarketing. Decision Support Systems, Elsevier, 62:22-31, June 2014
#'
#' @param root path to the data location
#' @param split string. 'train' or 'submission'
#' @param indexes set of integers for subsampling (e.g. 1:41188)
#' @param download whether to download or not
#' @param with_call_duration whether the call duration should be included as a feature. Could lead to leakage. Default: FALSE.
#'
#' @return A torch dataset that can be consumed with [torch::dataloader()].
#' @examples
#' if (torch::torch_is_installed() && FALSE) {
#' bank_mkt <- bank_marketing_dataset("./data", download = TRUE)
#' length(bank_mkt)
#' }
#' @export
bank_marketing_dataset <- torch::dataset(
  "BankMarketing",
  initialize = function(root, split = "train", indexes = NULL, download = FALSE, with_call_duration = FALSE) {

    # download ----------------------------------------------------------
    data_path <- maybe_download(
      url = "https://archive.ics.uci.edu/ml/machine-learning-databases/00222/bank-additional.zip",
      root = root,
      name = "bank-marketing",
      download = download,
      extract_fun = function(tmp, data_path) {
        unzip2(tmp, exdir = data_path)
      }
    )

    if(tolower(split) != "train") {
      stop("The bank marketing dataset only has a `train` split")
    }

    self$.path <- file.path(data_path, "bank-additional")

    dataset <- read.csv2(fs::path(data_path, "bank-additional/bank-additional-full.csv"))

    if (!with_call_duration)
      dataset <- dataset[,-which(colnames(dataset)=="duration")]

    # one-hot encode unordered categorical features

    unordered_categorical_features <- c("default",
                                        "job",
                                        "marital",
                                        "housing",
                                        "loan",
                                        "contact",
                                        "month",
                                        "day_of_week",
                                        "poutcome")
    for (catvar in unordered_categorical_features) {
      tmp_df <- model.matrix(~ 0 + as.data.frame(dataset)[,catvar])
      colnames(tmp_df) <- paste(catvar, levels(as.factor(as.data.frame(dataset)[,catvar])), sep = "_")
      dataset <- dataset[,-which(colnames(dataset)==catvar)]
      dataset <- cbind(dataset, tmp_df)
    }
    # encodes with integers the only ordered categorical feature, education

    educ_factors <- c("unknown",
                      "illiterate",
                      "basic.4y",
                      "basic.6y",
                      "basic.9y",
                      "high.school",
                      "professional.course",
                      "university.degree")
    educ <- factor(dataset[, "education"], order = TRUE, levels = educ_factors)
    dataset[, "education"] <- as.numeric(educ)
    dataset[, "y"] <- ifelse(dataset[, "y"] == "yes", 1, 0)

    # attributes the numbers to the data instance

    self$features <- as.matrix(dataset[,-which(colnames(dataset)=="y")])

    self$target <- dataset[,"y"]
  },

  .getitem = function(index) {

    force(index)

    x <- self$features[index, ]
    y <- self$target[index]

    x <- torch::torch_tensor(as.numeric(unlist(x)))
    y <- torch::torch_scalar_tensor(y)

    return(list(x = x, y = y))
  },

  .length = function() {
    nrow(self$features)
  }
)

#' Cityscapes Pix2Pix dataset
#'
#' Downloads and prepares the cityscapes dataset that has been used in the
#' [pix2pix paper](https://arxiv.org/abs/1611.07004).
#'
#' Find more information in the [project website](https://phillipi.github.io/pix2pix/)
#'
#' @inheritParams bird_species_dataset
#' @inheritParams torchvision::image_folder_dataset
#' @param ... Currently unused.
#'
#' @export
cityscapes_pix2pix_dataset <- torch::dataset(
  "CityscapesImagePairs",
  initialize = function(root, split = "train", download = FALSE, ...,
                        transform = NULL, target_transform = NULL) {

    url <- "http://efrosgans.eecs.berkeley.edu/pix2pix/datasets/cityscapes.tar.gz"

    data_path <- maybe_download(
      root = root,
      url = url,
      name = "cityscapes-image-pairs",
      download = download,
      extract_fun = function(f, exdir) {
        untar(f, exdir = exdir)
      }
    )

    self$split <- split

    path <- fs::path(
      data_path,
      "cityscapes",
      ifelse(self$split == "train", "train", "val")
    )

    self$files <- fs::dir_ls(path, glob = "*.jpg")
    self$transform <- if (is.null(transform)) identity else transform
    self$target_transform <- if (is.null(target_transform)) identity else target_transform
  },
  .getitem = function(i) {
    img <- jpeg::readJPEG(self$files[i])

    list(
      input_img = self$transform(img[,257:512,]),
      real_img = self$target_transform(img[,1:256,])
    )
  },
  .length = function() {
    length(self$files)
  }
)

#' Spam Dataset Loader
#'
#' Defines the spam dataset commonly used in machine learning.
#'
#' @param url A character string representing the URL of the dataset.
#' @param download Logical; whether to download the dataset. Defaults to FALSE.
#' @param transform Function to apply transformations to the features. Defaults to NULL.
#' @param target_transform Function to apply transformations to the labels. Defaults to NULL.
#' @return A `torch::dataset` object for the spam dataset.
#' @examples
#' \dontrun{
#' # Simple usage:
#' ds <- spam_dataset(download = TRUE)
#' ds[1]
#' }
#' @export
spam_dataset <- torch::dataset(
  name = "spam_dataset",
  
  initialize = function(
    url = "https://hastie.su.domains/ElemStatLearn/datasets/spam.data",
    download = FALSE,
    transform = NULL,
    target_transform = NULL
  ) {
    data_path <- tempfile(fileext = ".data")
    
    if (download) {
      download.file(url, data_path, mode = "wb")
    } else {
      data_path <- url
    }
    
    raw_spam_data <- read.table(data_path, header = FALSE)
    
    self$x_data <- as.matrix(raw_spam_data[, -ncol(raw_spam_data)])
    self$y_data <- as.numeric(raw_spam_data[, ncol(raw_spam_data)])
    
    self$transform <- transform
    self$target_transform <- target_transform
  },
  
  .getitem = function(index) {
    x <- self$x_data[index, ]
    y <- self$y_data[index]
    
    if (!is.null(self$transform)) {
      x <- self$transform(x)
    }
    
    if (!is.null(self$target_transform)) {
      y <- self$target_transform(y)
    }
    
    list(
      x = torch::torch_tensor(x, dtype = torch::torch_float()),
      y = torch::torch_tensor(y, dtype = torch::torch_long())
    )
  },
  
  .length = function() {
    nrow(self$x_data)
  }
)

#' Dog vs cats dataset
#'
#' Prepares the dog vs cats dataset available in Kaggle
#' [here](https://www.kaggle.com/c/dogs-vs-cats)
#'
#' @inheritParams guess_the_correlation_dataset
#' @param ... Currently unused.
#'
#' @return A [torch::dataset()] ready to be used with dataloaders.
#' @examples
#' if (torch::torch_is_installed() && FALSE) {
#' dogs_cats <- dogs_vs_cats_dataset("./data", token = "path/to/kaggle.json",
#'                                   download = TRUE)
#' length(dogs_cats)
#' }
#'
#' @importFrom torchvision base_loader
#' @importFrom stringr str_extract
#' @export
dogs_vs_cats_dataset <- torch::dataset(
  classes = c("dog", "cat"),
  initialize = function(root, split = "train", download = FALSE, ..., transform = NULL,
                        target_transform = NULL) {

    self$transform <- transform
    self$target_transform <- target_transform

    url <- "https://torch-cdn.mlverse.org/datasets/dogs-vs-cats.zip"

    data_path <- maybe_download(
      root = root,
      name = "dogs-vs-cats",
      url = url,
      download = download,
      extract_fun = function(temp, data_path) {
        unzip2(temp, exdir = data_path)
        unzip2(fs::path(data_path, "train.zip"), exdir = data_path)
        unzip2(fs::path(data_path, "test1.zip"), exdir = data_path)
        fs::file_delete(fs::path(data_path, "train.zip"))
        fs::file_delete(fs::path(data_path, "test1.zip"))
      }
    )

    if (!fs::dir_exists(data_path))
      cli::cli_abort("No data found. Please use `download = TRUE`.")

    if(split == "train") {
      self$images <- fs::dir_ls(fs::path(data_path, "train"))
    } else if(split == "test") {
      self$images <- fs::dir_ls(fs::path(data_path, "test1"))
    } else {
      cli::cli_abort(c(
        "Only 'train' and 'test' split are supported.",
        i = "Got {.str {split}}"
      ))
    }
    self$targets <- stringr::str_extract(
      fs::path_file(self$images),
      "[^.]+(?=\\.)"
    )
    self$targets <- match(self$targets, self$classes)
  },
  .getitem = function(i) {
    x <- base_loader(self$images[i])
    y <- self$targets[i]

    if (!is.null(self$transform))
      x <- self$transform(x)

    if (!is.null(self$target_transform))
      y <- self$target_transform(y)

    list(x, y)
  },
  .length = function() {
    length(self$images)
  }
)

test_that("cityscapes_pix2pix works", {

  root <- tempfile()

  train <- cityscapes_pix2pix_dataset(
    root = root,
    download = TRUE,
    transform = torchvision::transform_to_tensor,
    target_transform = torchvision::transform_to_tensor
  )

  valid <- cityscapes_pix2pix_dataset(
    root = root,
    split = "valid",
    download = FALSE,
    transform = torchvision::transform_to_tensor,
    target_transform = torchvision::transform_to_tensor
  )

  expect_tensor_shape(train[1][[1]], c(3, 256, 256))
  expect_tensor_shape(train[1][[2]], c(3, 256, 256))

  expect_tensor_shape(valid[1][[1]], c(3, 256, 256))
  expect_tensor_shape(valid[1][[2]], c(3, 256, 256))
})

test_that("spam_dataset works as expected", {
  dataset <- tryCatch(
    spam_dataset(download = TRUE),
    error = function(e) {
      if (grepl("cannot open URL|HTTP status|download.file", conditionMessage(e)))
        skip(paste("Spam dataset download failed:", conditionMessage(e)))
      stop(e)
    }
  )

  expect_true(length(dataset) > 0)

  item <- dataset[1]
  expect_named(item, c("x", "y"))
  expect_equal(length(item$x), 57)
  expect_true(as.integer(item$y) %in% c(0, 1))
})

test_that("oxford flowers dataset", {
  skip_if_not_installed("R.matlab")

  root <- tempfile()

  train <- oxford_flowers102_dataset(
    root = root,
    download = TRUE,
    transform = torchvision::transform_to_tensor
  )

  valid <- oxford_flowers102_dataset(
    root = root,
    split = "valid",
    download = FALSE,
    transform = torchvision::transform_to_tensor
  )

  test <- oxford_flowers102_dataset(
    root = root,
    split = "test",
    download = FALSE,
    transform = torchvision::transform_to_tensor
  )

  all <- oxford_flowers102_dataset(
    root = root,
    split = c("train", "valid", "test"),
    download = FALSE,
    transform = torchvision::transform_to_tensor
  )

  expect_equal(train$classes[train[1]$y], "pink primrose")

  expect_equal(length(all), 8189)
  expect_equal(length(valid), 1020)
  expect_equal(length(train), 1020)
  expect_equal(length(test), 6149)

  expect_tensor_shape(train[1][[1]], c(3, 500, 754))
  expect_tensor_shape(valid[1][[1]], c(3, 500, 606))
  expect_tensor_shape(all[1][[1]], c(3, 500, 754))
})

test_that("imdb dataset works", {

  tmp <- tempfile()

  dataset <- imdb_dataset(
    root = tmp,
    download = TRUE,
    num_words = 5000
  )

  expect_equal(length(dataset), 25000)

  # can used the cached dataset
  dataset <- imdb_dataset(
    root = tmp,
    download = TRUE,
    num_words = 3000,
    maxlen = 2500
  )

  expect_equal(length(dataset), 25000)
  expect_equal(dataset[1]$x[1], 1)
  expect_equal(length(dataset[1]$x), 2500)

  # can load a batch of obs
  dl <- torch::dataloader(dataset, batch_size = 32)
  x <- coro::collect(dl, 1)[[1]]

  expect_tensor_shape(x$x, c(32, 2500))
  expect_tensor_shape(x$y, c(32))

  # can load tests dataset
  dataset <- imdb_dataset(
    root = tmp,
    download = TRUE,
    num_words = 5000,
    split = "test"
  )

  expect_equal(length(dataset), 25000)

})

is_torch_tensor <- function(x) {
  inherits(x, "torch_tensor")
}

expect_no_error <- function(object, ...) {
  expect_error(object, NA, ...)
}

expect_tensor_shape <- function(object, expected) {
  expect_tensor(object)
  expect_equal(object$shape, expected)
}

expect_tensor <- function(object) {
  expect_true(is_torch_tensor(object))
  expect_no_error(torch::as_array(object))
}

expect_equal_to_r <- function(object, expected) {
  expect_equal(torch::as_array(object), expected)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oxford-pet-dataset.R
\name{oxford_pet_dataset}
\alias{oxford_pet_dataset}
\title{Oxford Pet Dataset}
\usage{
oxford_pet_dataset(
  root,
  split = "train",
  target_type = c("trimap", "species", "breed"),
  download = FALSE,
  ...,
  transform = NULL,
  target_transform = NULL
)
}
\arguments{
\item{root}{path to the data location}

\item{split}{train, test or valid}

\item{target_type}{The type of the target:
\itemize{
\item 'trimap': returns a mask array with one class per pixel.
\item 'species': returns the species id. 1 for cat and 2 for dog.
\item 'breed': returns the breed id. see \code{dataset$breed_classes}.
}}

\item{download}{wether to download or not}

\item{...}{Currently unused.}

\item{transform}{A function/transform that takes in an PIL image and returns
a transformed version. E.g, \code{\link[torchvision:transform_random_crop]{transform_random_crop()}}.}

\item{target_transform}{A function/transform that takes in the target and
transforms it.}
}
\description{
The Oxford-IIIT Pet Dataset is a 37 category pet dataset with roughly
200 images for each class. The images have a large variations in scale,
pose and lighting. All images have an associated ground truth annotation of
species (cat or dog), breed, and pixel-level trimap segmentation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/imdb.R
\name{imdb_dataset}
\alias{imdb_dataset}
\title{IMDB movie review sentiment classification dataset}
\usage{
imdb_dataset(
  root,
  download = FALSE,
  split = "train",
  shuffle = (split == "train"),
  num_words = Inf,
  skip_top = 0,
  maxlen = Inf,
  start_char = 2,
  oov_char = 3,
  index_from = 4
)
}
\arguments{
\item{root}{path to the data location}

\item{download}{wether to download or not}

\item{split}{train, test or valid}

\item{shuffle}{whether to shuffle or not the dataset. \code{TRUE} if \code{split=="train"}}

\item{num_words}{Words are ranked by how often they occur (in the training set),
and only the num_words most frequent words are kept. Any less frequent word
will appear as oov_char value in the sequence data. If \code{Inf}, all words are
kept. Defaults to None, so all words are kept.}

\item{skip_top}{skip the top N most frequently occurring words (which may not be informative).
These words will appear as oov_char value in the dataset. Defaults to 0, so
no words are skipped.}

\item{maxlen}{int or \code{Inf}. Maximum sequence length. Any longer sequence will
be truncated. Defaults to Inf, which means no truncation.}

\item{start_char}{The start of a sequence will be marked with this character.
Defaults to 2, because 1 is usually the padding character.}

\item{oov_char}{int. The out-of-vocabulary character. Words that were cut out
because of the num_words or skip_top limits will be replaced with this character.}

\item{index_from}{int. Index actual words with this index and higher.}
}
\description{
The format of this dataset is meant to replicate that provided by
\href{https://keras.io/api/datasets/imdb/}{Keras}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bird-species.R
\name{bird_species_dataset}
\alias{bird_species_dataset}
\title{Bird species dataset}
\usage{
bird_species_dataset(root, split = "train", download = FALSE, ...)
}
\arguments{
\item{root}{path to the data location}

\item{split}{train, test or valid}

\item{download}{wether to download or not}

\item{...}{other arguments passed to \code{\link[torchvision:image_folder_dataset]{torchvision::image_folder_dataset()}}.}
}
\value{
A \code{\link[torch:dataset]{torch::dataset()}} ready to be used with dataloaders.
}
\description{
Downloads and prepares the 450 bird species dataset found on Kaggle.
}
\examples{
if (torch::torch_is_installed() && FALSE) {
birds <- bird_species_dataset("./data", token = "path/to/kaggle.json",
                              download = TRUE)
length(birds)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spam-dataset.R
\name{spam_dataset}
\alias{spam_dataset}
\title{Spam Dataset Loader}
\usage{
spam_dataset(
  url = "https://hastie.su.domains/ElemStatLearn/datasets/spam.data",
  download = FALSE,
  transform = NULL,
  target_transform = NULL
)
}
\arguments{
\item{url}{A character string representing the URL of the dataset.}

\item{download}{Logical; whether to download the dataset. Defaults to FALSE.}

\item{transform}{Function to apply transformations to the features. Defaults to NULL.}

\item{target_transform}{Function to apply transformations to the labels. Defaults to NULL.}
}
\value{
A \code{torch::dataset} object for the spam dataset.
}
\description{
Defines the spam dataset commonly used in machine learning.
}
\examples{
\dontrun{
# Simple usage:
ds <- spam_dataset(download = TRUE)
ds[1]
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bank-marketing.R
\name{bank_marketing_dataset}
\alias{bank_marketing_dataset}
\title{Bank marketing dataset}
\usage{
bank_marketing_dataset(
  root,
  split = "train",
  indexes = NULL,
  download = FALSE,
  with_call_duration = FALSE
)
}
\arguments{
\item{root}{path to the data location}

\item{split}{string. 'train' or 'submission'}

\item{indexes}{set of integers for subsampling (e.g. 1:41188)}

\item{download}{whether to download or not}

\item{with_call_duration}{whether the call duration should be included as a feature. Could lead to leakage. Default: FALSE.}
}
\value{
A torch dataset that can be consumed with \code{\link[torch:dataloader]{torch::dataloader()}}.
}
\description{
Prepares the Bank marketing dataset available on UCI Machine Learning repository \href{https://archive.ics.uci.edu/ml/datasets/Bank+Marketing}{here}
The data is available publicly for download, there is no need to authenticate.
Please cite the data as Moro et al., 2014
S. Moro, P. Cortez and P. Rita. A Data-Driven Approach to Predict the Success of Bank Telemarketing. Decision Support Systems, Elsevier, 62:22-31, June 2014
}
\examples{
if (torch::torch_is_installed() && FALSE) {
bank_mkt <- bank_marketing_dataset("./data", download = TRUE)
length(bank_mkt)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oxford-flowers-dataset.R
\name{oxford_flowers102_dataset}
\alias{oxford_flowers102_dataset}
\title{102 Category Flower Dataset}
\usage{
oxford_flowers102_dataset(
  root,
  split = "train",
  target_type = c("categories"),
  download = FALSE,
  ...,
  transform = NULL,
  target_transform = NULL
)
}
\arguments{
\item{root}{path to the data location}

\item{split}{train, test or valid}

\item{target_type}{Currently only 'categories' is supported.}

\item{download}{wether to download or not}

\item{...}{Currently unused.}

\item{transform}{A function/transform that takes in an PIL image and returns
a transformed version. E.g, \code{\link[torchvision:transform_random_crop]{transform_random_crop()}}.}

\item{target_transform}{A function/transform that takes in the target and
transforms it.}
}
\description{
The Oxford Flower Dataset is a 102 category dataset, consisting of 102 flower
categories. The flowers chosen to be flower commonly occuring in the United
Kingdom. Each class consists of between 40 and 258 images. The details of the
categories and the number of images for each class can be found on
\href{https://www.robots.ox.ac.uk/\%7Evgg/data/flowers/102/categories.html}{this category statistics page}.
}
\details{
The images have large scale, pose and light variations. In addition, there are
categories that have large variations within the category and several very
similar categories. The dataset is visualized using isomap with shape and colour
features.

You can find more info in the dataset \href{https://www.robots.ox.ac.uk/\%7Evgg/data/flowers/102/}{webpage}.
}
\note{
The official splits leaves far too many images in the test set. Depending
on your work you might want to create different train/valid/test splits.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guess-the-correlation.R
\name{guess_the_correlation_dataset}
\alias{guess_the_correlation_dataset}
\title{Guess The Correlation dataset}
\usage{
guess_the_correlation_dataset(
  root,
  split = "train",
  transform = NULL,
  target_transform = NULL,
  indexes = NULL,
  download = FALSE
)
}
\arguments{
\item{root}{path to the data location}

\item{split}{string. 'train' or 'submission'}

\item{transform}{function that takes a torch tensor representing an image and return another tensor, transformed.}

\item{target_transform}{function that takes a scalar torch tensor and returns another tensor, transformed.}

\item{indexes}{set of integers for subsampling (e.g. 1:140000)}

\item{download}{whether to download or not}
}
\value{
A torch dataset that can be consumed with \code{\link[torch:dataloader]{torch::dataloader()}}.
}
\description{
Prepares the Guess The Correlation dataset available on Kaggle \href{https://www.kaggle.com/c/guess-the-correlation}{here}
A copy of this dataset is hosted in a public Google Cloud
bucket so you don't need to authenticate.
}
\examples{
if (torch::torch_is_installed() && FALSE) {
gtc <- guess_the_correlation_dataset("./data", download = TRUE)
length(gtc)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cityscapes-pix2pix.R
\name{cityscapes_pix2pix_dataset}
\alias{cityscapes_pix2pix_dataset}
\title{Cityscapes Pix2Pix dataset}
\usage{
cityscapes_pix2pix_dataset(
  root,
  split = "train",
  download = FALSE,
  ...,
  transform = NULL,
  target_transform = NULL
)
}
\arguments{
\item{root}{path to the data location}

\item{split}{train, test or valid}

\item{download}{wether to download or not}

\item{...}{Currently unused.}

\item{transform}{A function/transform that takes in an PIL image and returns
a transformed version. E.g, \code{\link[torchvision:transform_random_crop]{transform_random_crop()}}.}

\item{target_transform}{A function/transform that takes in the target and
transforms it.}
}
\description{
Downloads and prepares the cityscapes dataset that has been used in the
\href{https://arxiv.org/abs/1611.07004}{pix2pix paper}.
}
\details{
Find more information in the \href{https://phillipi.github.io/pix2pix/}{project website}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dogs-vs-cats.R
\name{dogs_vs_cats_dataset}
\alias{dogs_vs_cats_dataset}
\title{Dog vs cats dataset}
\usage{
dogs_vs_cats_dataset(
  root,
  split = "train",
  download = FALSE,
  ...,
  transform = NULL,
  target_transform = NULL
)
}
\arguments{
\item{root}{path to the data location}

\item{split}{string. 'train' or 'submission'}

\item{download}{whether to download or not}

\item{...}{Currently unused.}

\item{transform}{function that takes a torch tensor representing an image and return another tensor, transformed.}

\item{target_transform}{function that takes a scalar torch tensor and returns another tensor, transformed.}
}
\value{
A \code{\link[torch:dataset]{torch::dataset()}} ready to be used with dataloaders.
}
\description{
Prepares the dog vs cats dataset available in Kaggle
\href{https://www.kaggle.com/c/dogs-vs-cats}{here}
}
\examples{
if (torch::torch_is_installed() && FALSE) {
dogs_cats <- dogs_vs_cats_dataset("./data", token = "path/to/kaggle.json",
                                  download = TRUE)
length(dogs_cats)
}

}
